#ifndef __VSV3_CLIENT_API_H_INCLUDED_SW__
#define __VSV3_CLIENT_API_H_INCLUDED_SW__

#include "system.h"
#include <videosystem3/platform.h>
#include <videosystem3/v3clientlib-tineincludewrapper.h>
#include <stdint.h> // for uint32_t 

#ifdef __cplusplus
extern "C" {
#endif

    //
    // Most simple structure which can contain an arbitrary number of video
    // images (tine DIMAGE format type) which may or may not form a 
    // sequence (see vsIsSequence() and vsIsCompleteSequence() ).
    //
    // The structure is used in order to pass and return a bunch of images 
    // across API functions.
    //
    typedef struct
    {
        // 1. structure is considered empty (no video frames inside) if 
        //    numFrames member is zero
        // 2. value of imageArray is undefined if numFrames member is zero
        // 3. if numFrames is >0, then imageArray must be initialized 
        //    properly (points to a memory location big enough to store the
        //    header fields of 'numFrames' video images
        // 4. if imageArray pointer is initializied this does NOT imply that
        //    individual header fields of each DIMAGE substructure are 
        //    properly initialized
        // 5. for each individual DIMAGE, frameBuffer member is either NULL
        //    (not initialized) or not NULL (initialized->means that memory 
        //    was allocated (and can be freed if necessary)

        DIMAGE *imageArray; // DIMAGE declared in TINE, see tinetype.h
        UINT32 numFrames;   // number of frames in sequence, zero in case 
        // of no frame
    } VSBUNCHOFIMAGES;

    //
    // This function will initialize the structure VSBUNCHOFIMAGES in a 
    // recommended way.
    //
    // Remark: 
    //   There is no checking whether the structure is already 
    //   initialized or memory has been allocated before. This function is 
    //   only meant to be used on _uninitialized_ structures in order to 
    //   get a defined initial value of its members.
    //
    // Return code: 
    //   0 - failure (aImages passed NULL)
    //   1 - success
    //
    VIDEOSYSTEM3_API int vsInitializeBunchStructure
        ( 
        VSBUNCHOFIMAGES *aImages 
        );

    
    // 
    // This function initializes the structure VSBUNCHOFIMAGES so
    // that it can contain up to aiNumFrames number of images. Memory
    // will be allocated for that.
    //
    // Remarks: 
    //   There is no checking whether the structure is already 
    //   initialized or memory has been allocated before. 
    //   
    // Return code:
    //   0 - failure
    //   1 - success
    //

    VIDEOSYSTEM3_API int vsCreateBunchStructure    // vsCBS: DRAFT, introduced Sept 24, 2015
        ( 
        VSBUNCHOFIMAGES *aImages,
        unsigned int aiNumFrames,
        unsigned int aiFrameBufferSizeBytes       // aiFBSB: introduced Dec 22, 2015
        );

    

    //
    // This function will clean (deallocate) everything what's possible 
    // inside aImages (memory portion of each individual image) as well as 
    // aImages memory buffer and structure members itself (using C lib 
    // free() function). On return, all memory is freed and the structure 
    // is returned uninitialized.
    //
    // Remark:
    //   If an uninitialized 'aImages' is passed, the function may not work 
    //   as expected and must be considered harmful in this case!
    //
    // Return code: 
    //   0 - failure (aImages passed NULL)
    //   1 - success
    //
    VIDEOSYSTEM3_API int vsFreeBunch
        ( 
        VSBUNCHOFIMAGES *aImages 
        );


    //
    // This function estimates the maximum video image (frame) size in bytes. 
    // In order to do this, the TINE video image property will be queried
    // for its data type and data type size. In any case of error, default 
    // buffer size of AVINE is returned. The default buffer size is specified at
    // (<videosystem3-general/globalheaders.h> #define MAX_TRANSPORT_SIZE). 
    // Currently (Dec 22, 2015), MAX_TRANSPORT_SIZE is set to 9*1024*1024 (9 MB). 
    //
    // Parameters:
    //  asTineAddress - zero-terminated string containing address of tine 
    //                 server, e.g. "/PITZ/FG1.Analogue/Output". It will
    //                 be passed unmodified to TINE API function 
    //                 GetTargetPropertyInformation(), parameter 'srv'.
    //
    //  asTinePropertyName - Zero-terminated string which contains the name
    //                      of TINE property, e.g. "Frame.Sched".
    //                      If NULL, "Frame" is used as a default.
    //
    // Return value: 
    //   The number of bytes recommended as frame buffer. The number is always positive.
    //   Observed values, as of Dec 2015, are: 6 MB to 33 MB.
    //
    // Remark:
    //   1. The function was introduced Dec 22, 2015.
    //

    VIDEOSYSTEM3_API int vsEstimateMaxFrameSizeBytes
        ( 
        const char *asTineAddress, 
        const char *asTinePropertyName
        );
    

    //
    // This function will receive aNumFrames from a VSv3 server equipped 
    // with VSv3 output interface (usually it can be recognized by TINE 
    // device 'Output' and the presence of TINE property names 'Frame' 
    // and 'Frame.Sched' below). The server is addressed by aTineAddress
    // and aTineProperty. Received frames will be put into aImages 
    // structure. If no error or timeout occurs, aNumFrames video frames
    // will be returned.
    //
    // Warning: No checking is performed here whether all frames came from
    // the same image source or whether they form a complete sequence (no 
    // gap as depicted by incremented-by-1-for-each-subsequent-video-frame
    // video frame number). There are special functions provided which 
    // perform such checks (see vsIsSequence(), vsIsCompleteSequence() ).
    //
    // Parameters:
    //  aImages - [InOut] structure where the received video frames will 
    //            be put one after another, the structure must be passed 
    //            uninitialized (numFrames = 0). Memory blocks used to 
    //            store video image header(s) as well as video image bits 
    //            will be allocated internally (using C lib malloc() ).
    //
    //  aTineAddress - zero-terminated string containing address of tine 
    //                 server, e.g. "/PITZ/FG1.Analogue/Output". It will
    //                 be passed unmodified to TINE API function 
    //                 AttachLinkEx2(), parameter devName.
    //
    //  aTinePropertyName - Zero-terminated string which contains the name
    //                      of TINE property, e.g. "Frame.Sched".
    //                      This parameter will be passed unmodified to TINE
    //                      API function AttachLinkEx2(), parameter 
    //                      devProperty.
    //
    //  aNumFrames - Number of frames to get.
    //               The recommended interval is [1..100]. There are no
    //               checks, but if you want to grab more than 100 frames
    //               at once you should know what you're doing, what has
    //               to be taken care of and what could be possible side 
    //               effects.
    //
    //  aTineTimeoutMilliseconds - TINE access rate parameter for asynch-
    //                             ronous link to server which is created 
    //                             internally. It will be passed unmodified
    //                             to TINE API function AttachLinkEx2(), 
    //                             parameter 'pollingRate'. In most cases, 
    //                             the recommended value of 1000 ms one can
    //                             just pass and it will be fine. The 
    //                             advised range is 300 to 4000 ms.
    //
    //  aTineMode - TINE data transfer mode of asynchronous link to server
    //              which is created internally. It will be passed 
    //              unmodified to TINE API function AttachLinkEx2(), 
    //              parameter 'mode'. Please note that 
    //              TINE_DECORATED_CONSTANTS are used (TDC_).
    //              The following combinations are recommended:
    //                  a) TDC_CM_TIMER|TDC_CM_SYNCNOTIFY|TDC_CM_CONNECT
    //                  b) TDC_CM_EVENT|TDC_CM_SYNCNOTIFY|TDC_CM_MCAST 
    //
    //              Note: These combinations are under constant evaluation 
    //              and may change!
    //
    // Return code: 
    //   1 - success, aNumFrames video images successfully put into aImages
    //       structure
    //   0 - parameter error (aImages NULL, malfored aTineAddress, illegal 
    //       aNumFrames value, aImages structure is not "empty")
    //  -1 - timeout occurred
    //  -2 - (memory allocation error)
    //  -3 - (some bad tine error code happened during transfer)
    //  -4 - unknown error
    //  -5 - tine error at attachlink (this means transfer could not even started)
    //
    // Remarks:
    //   1. The function blocks until all frames have been received, a
    //      timeout has occurred or an error condition was met.
    //   2. Memory will be allocated in order to keep the received video 
    //      frames. References to memory blocks will be put to aImages 
    //      structure.
    //   3. Some errors may be printed to stderr.
    //   4. Internal timeout is set to 5 seconds (any 5 seconds there 
    //      must be some answer by TINE kernel).
    //
    VIDEOSYSTEM3_API int vsGrabFrames
        (
        VSBUNCHOFIMAGES *aImages, 
        const char *aTineAddress, 
        const char *aTinePropertyName, 
        unsigned int aNumFrames, 
        int aTineTimeoutMilliseconds, 
        int aTineMode 
        );


    //
    // This function differs from vsGrabFrames() in the following way:
    //  a) the function can be stopped by the user (parameter apiInterrupt)
    //  b) the user can get to know how many percent of the grabbing process 
    //     has been finished already 
    //  c) the function's return value will indicate the number of frames it 
    //     could successfully grab 
    //  d) a separate error parameter is there to indicate an error
    //
    // In the following, only the differences to vsGrabFrames will be 
    // pointed out:
    //
    //
    // Parameters:
    //
    //  apiOutError - pointer to an integer which the user needs to check
    //                after function returns. It may contain error code.
    //                This parameter cannot be NULL.
    //           
    //             possible values:
    //               0 - no error
    //              -1 - parameter error (aImages NULL, malfored aTineAddress,
    //                   illegal NumFrames value, aImages structure is not 
    //                   "empty")
    //              -2 - timeout occurred
    //              -3 - memory allocation error
    //              -4 - some bad tine error code happened during transfer
    //              -5 - unknown error
    //              -6 - tine error at attachlink (transfer could not even 
    //                   start)
    //              -7 - user break (not really an error)
    //              -8 - semaphore lock/unlock error (added Sept 2, 2013)
    //              -9 - unable to create semaphore (added Sept 2, 2013)
    //
    //  apiOutTineError - If some bad tine error happened, it will be returned here. 
    //                    Most probably but not limited to return code -4 or -6,
    //                    if some bad tine error happened, it can be obtained
    //                    using this output variable parameter.
    //                    This parameter can be NULL.                
    //
    //
    //  apiInInterrupt - pointer to an integer which the user needs to 
    //                 set to '1' in order to break the function during
    //                 execution (cancel grabbing attempt but return 
    //                 frames which have been grabbed up to now)
    //                 NOTE: The parameter can be NULL. In case the 
    //                 parameter is not NULL, user has to set the 
    //                 integer to some value which is not 1, before 
    //                 calling the function. Otherwise it will break for sure.
    // 
    //  apdOutPercentageFinished - if this parameter is not NULL, the function
    //                             will update the percentage (0 (0%) .. 1 (100%)) 
    //                             of number of frames already received in 
    //                             comparison to  the number of frames requested.
    //
    //
    // Return value: 
    //               0 - no frame could be grabbed
    //   1..aNumFrames - number of frames that have been grabbed (and returned 
    //                   inside aImages structure)
    //   
    //
    VIDEOSYSTEM3_API int vsGrabFramesEx
        (
        VSBUNCHOFIMAGES *aImages, 
        const char *aTineAddress, 
        const char *aTinePropertyName, 
        unsigned int aNumFrames, 
        int aTineTimeoutMilliseconds, 
        int aTineMode,
        int *apiOutError,
        int *apiOutTineError,
        int *apiInInterrupt,
        double *apdOutPercentageFinished
        );


    // SW Apr 29, 2013 1/4 vsGrabFramesEx2 is an idea
    // SW Apr 29, 2013 2/4 added: aImagesDOut (for system, user and tine timestamp)
    // SW Apr 29, 2013 3/4 aImagesDOut can be NULL (in this way it is neither touched nor filled)
    // SW Apr 29, 2013 4/4 user must properly allocate and deallocate aImagesDOut
    //
    // SW May  7, 2013 1/1 aImagesDOut: array to aNumFrames DTYPE structures
    //
    VIDEOSYSTEM3_API int vsGrabFramesEx2
        (
        VSBUNCHOFIMAGES *aImages, 
        DTYPE *aImagesDOut,
        const char *aTineAddress, 
        const char *aTinePropertyName, 
        unsigned int aNumFrames, 
        int aTineTimeoutMilliseconds, 
        int aTineMode,
        int *apiOutError,
        int *apiOutTineError,
        int *apiInInterrupt,
        double *apdOutPercentageFinished
        );


    // vsGrabFrame is meant for acquiring single video frame, either 
    // occasionally or repetitive in a loop. It uses TINE's alsnExecLinkEx 
    // under the hood.
    //
    // Parameters:
    //
    //  apInOutImage - on successful return, the video frame (image) will be  
    //                 returned here. The DIMAGE structure has to be 
    //                 initialized properly before put here (fields frameBuffer 
    //                 and frameBufferSize needs to be initialized, see remarks)
    //
    //  apDOut - if not NULL, the data fields of TINE's DOUT, which may contain 
    //           important information (time stamp, system stamp) returned by
    //           video server, are passed along here, on successful return code
    //           SW Note added Jan 27, 2014: the dout variable passed to apDOut does
    //           not need to be DTYPEZERO'd.
    //
    //  asTineAddress - zero-terminated string containing address of TINE 
    //                  server, e.g. "/PITZ/FG1.Analogue/Output". It will
    //                  be passed unmodified to TINE API function 
    //                  alsnExecLinkEx(), parameter devName.
    //
    //  asTinePropertyName - zero-terminated string which contains the name
    //                       of TINE property, e.g. "Frame.Sched".
    //                       This parameter will be passed unmodified to TINE
    //                       API function alsnExecLinkEx(), parameter 
    //                       devProperty.
    //
    //  aiTineTimeoutMilliseconds - TINE access rate parameter for link to 
    //                              server which is created internally. 
    //                              It will be passed unmodified
    //                              to TINE API function alsnExecLinkEx(), 
    //                              parameter 'timeout'. In most cases, 
    //                              the recommended value of 1000 ms one can
    //                              just pass and it will be fine. The 
    //                              advised range is 300 to 4000 ms.
    //
    //  aiTineMode - TINE data transfer mode of link to server
    //               which is created internally. It will be passed 
    //               unmodified to TINE API function alsnExecLinkEx(), 
    //               parameter 'mode'. Please note that 
    //               TINE_DECORATED_CONSTANTS are used (TDC_).
    //               The following combinations are recommended:
    //                   a) TDC_CA_READ|TDC_CA_SYNCNOTIFY|TDC_CA_CONNECT
    //                   b) TDC_CA_READ|TDC_CA_SYNCNOTIFY|TDC_CA_MCAST
    //
    //               Notes:
    //                - In case TDC_CA_MCAST is unknown, please use its synonym 
    //                  TDC_CA_NETWORK.
    //                - These combinations are under constant evaluation 
    //                  and may change!
    //
    //
    //  apiOutTineReturnCode - If some tine error happened, it will be 
    //                         returned here. Most probably but not 
    //                         limited to return code -4 or -6, if some 
    //                         bad tine error happened, it can be 
    //                         obtained using this output variable 
    //                         parameter.
    //                         This parameter can be NULL.
    //
    // Return code: 
    //   1 - success, video frame received successfully 
    //   0 - parameter error (aImages NULL, malfored aTineAddress, illegal 
    //       aNumFrames value, aImages structure is not "empty")
    //  -1 - (some bad tine error code happened during transfer)
    //  -2 - image did not pass validity check!
    //  -3 - some bad tine error code happened during property query
    //  -4 - asTineAddress parameter is too short (<5 characters) or too long (>132 characters if first character is a '/', >131 characters otherwise)
    //  -5 - internal error
    //
    // Remarks:
    //  - There are functions available inside the library in order to 
    //    create/free an appropriate apInOutImage. Please consult 
    //    vsInitializeImage() and vsFreeImage(). In case one wants to
    //    initialize an appropriate DIMAGE by hand, the following 
    //    pseudocode shows how it's done:
    //
    //    {
    //      // MAX_TRANSPORT_SIZE=1024*1024*9 at Jan 2, 2014
    //      const int ciFrameBufferSize = 1024*1024*9; 
    //      DIMAGE img;
    //
    //      char videoFrameBuffer[ciFrameBufferSize]; // char: 1 byte
    //      img.frameBuffer = &videoFrameBuffer;
    //      img.frameBufferSize = ciFrameBufferSize;
    //
    //      int ret = vsGrabFrame( &img, ...)
    //    }
    // 
    //    Please note that only in case vsInitializeImage is used, vsFreeImage 
    //    can (and should) be used to free memory!
    //
    //
    VIDEOSYSTEM3_API int vsGrabFrame
        ( 
        DIMAGE *apInOutImage, 
        DTYPE *apDOut,
        const char *asTineAddress,
        const char *asTinePropertyName,
        int aiTineTimeoutMilliseconds,
        int aiTineMode,
        int *apiOutTineError 
        );


    // This function will initialize (and allocate) memory buffer for one 
    // image.
    //
    // Return variable:
    //
    //  NULL - error initializing (and allocating) memory, usually this means
    //         out of memory
    //
    //  non-NULL - pointer to video image
    //
    //
    // Remark:
    //  Please note that memory has been allocated which needs to be manually 
    //  freed using library function vsFreeImage() later on.
    // 
    VIDEOSYSTEM3_API DIMAGE *vsInitializeImage();

    /*
    //
    // This function will initialize (and allocate) memory buffer for one 
    // image.
    //
    // Return variable:
    //
    //  NULL - error initializing (and allocating) memory, usually this means
    //         out of memory
    //
    //  non-NULL - pointer to video image
    //
    //
    // Remarks:
    //  1. Please note that memory has been allocated which needs to be manually 
    //     freed using library function vsFreeImage() later on.
    //  2. The function was introduced Dec 22, 2015.
    // 
    VIDEOSYSTEM3_API DIMAGE *vsInitializeImageEx
        (
        int aiFrameBufferSizeBytes
        );
        */

    // This function will free memory previously allocated by a successful 
    // call to vsInitializeImage() function.
    //
    // Parameter:
    //
    //   apImage - pointer to DIMAGE structure previously returned by 
    //             vsInitializeImage 
    //
    //
    // Return code:
    //  
    //   1 - success, memory has been freed
    //   0 - parameter error, no memory has been freed
    //  -1 - invalid DIMAGE passed (DIMAGE.frameBuffer is NULL),
    //       memory which could be freed has been freed
    //
    //
    // Remark:
    //  There is a danger if the field DIMAGE.frameBuffer has been changed 
    //  in between vsInitializeImage() ... vsFreeImage() calls. In that case
    //  it is recommended to set DIMAGE.frameBuffer to NULL before calling 
    //  vsFreeImage() and ignore return code -1.
    // 
    VIDEOSYSTEM3_API int vsFreeImage
        ( 
        DIMAGE *apImage 
        );



    //
    // This function checks whether all images passed in the bunch structure
    // are part of a simple sequence (simple: may contain gaps):
    //
    //  - all images are valid by itself (consistent header fields, 
    //    no "rubbish")
    //  - all images come from the same image source
    //  - all images contain mainly the same header fields (like width,
    //    height, aoi, scale etc.)
    //  - the framenumber is increasing (but there may be gaps in frame number
    //    (see also 'isCompleteSequence()')
    //
    // Parameter:
    //	aImages - a properly initialized and filled structure containing 
    //            a bunch of video frames
    //
    // Return code: 
    //   1 - yes, it is sequence
    //   0 - no, it is not a sequence (either because the bunch is empty
    //       or the images in the bunch do not form a sequence)
    //  -1 - error on analysing whether it is sequence or not (bad 
    //       parameters or bad datastructures behind parameters)
    //
    // Remarks:
    //   1. A single valid frame is the smallest possible sequence (1 is re-
    //      turned in this case)
    //
    VIDEOSYSTEM3_API int vsIsSequence
        ( 
        const VSBUNCHOFIMAGES * const aImages
        );

    //
    // This function checks whether all images passed in the bunch structure
    // are part of a sequence without gaps:
    //
    //  - all images are valid by itself (consistent header fields, 
    //    no "rubbish")
    //  - all images come from the same image source
    //  - all images contain mainly the same header fields (like width,
    //     height, aoi, scale etc.)
    //  - the framenumber is increasing one by one (considered complete 
    //    sequence aka. sequence without gaps)
    //  
    // Parameter:
    //	aImages - a properly initialized and filled structure containing
    //            a bunch of video frames
    //
    // Return code: 
    //   1 - yes, it is a complete sequence (no gaps)
    //   0 - no, it is not a complete sequence (it is a sequence but it
    //       contains gaps)
    //  -1 - error on analysing whether it is sequence or not (bad para-
    //       meters or bad datastructures behind parameters)
    //
    // Remarks:
    //   1. vsIsSequence() is called internally. This function just adds
    //      more checking afterwards.
    //   2. A single valid frame is the smallest complete sequence (1 is 
    //      returned in this case).
    //
    VIDEOSYSTEM3_API int vsIsCompleteSequence
        ( 
        const VSBUNCHOFIMAGES * const aImages 
        );

    //
    // If there is some compressed image format inside DIMAGE aIn, a cloned
    // image object will be created (aOut), which contains uncompressed raw 
    // video bits instead of compressed bits. Currently, only JPEG -> Raw
    // (24 bits per pixel colour jpeg -> raw RGB24 and 8bits per pixel 
    // grayscale jpeg -> raw grayscale using 1 byte per pixel) is supported.
    // 
    // Parameters:
    //
    //  aIn  - well-filled DIMAGE header containing compressed image
    //  aOut - on successful return, a cloned aIn image with uncompressed 
    //         image bits will be returned here
    //
    // Return code:
    //
    //   1 - success 
    //   0 - failure - image was not compressed
    //  -1 - failure - error on decompression (memory alloc error)
    //  -2 - failure - internal error decompressing jpeg
    //  -3 - failure - parameter error (i.e. NULL pointer passed)
    //
    // Remark:
    //  1. Only in case function returns 1, aOut will contain a valid image 
    //     (with frame buffer memory just allocated and filled in)
    //
    VIDEOSYSTEM3_API int vsDecompressImage
        (
        DIMAGE *aOut, 
        const DIMAGE * const aIn
        );


    //
    // If there is some compressed image format inside DIMAGE apInOut,
    // the compressed contents can be uncompressed using this function.
    // The uncompressed data will be returned in the same data buffer.
    // 
    // Currently, only JPEG -> Raw (24 bits per pixel colour jpeg -> raw RGB24
    // and 8bits per pixel grayscale jpeg -> raw grayscale using 1 byte per 
    // pixel) is supported.
    // 
    // Parameters:
    //
    //  apInOut  - on input: well-filled DIMAGE header containing compressed 
    //                       image 
    //             on successful return: uncompressed image will be returned 
    //                                   here
    //
    //
    // Return code:
    //
    //   1 - success 
    //   0 - failure - image was not compressed
    //  -1 - failure - error on decompression (memory alloc error)
    //  -2 - failure - internal error decompressing jpeg
    //  -3 - failure - parameter error (i.e. NULL pointer passed)
    //  -4 - failure - image buffer not big enough for uncompressed video image
    //
    // Remark:
    //  In any unsuccessful return code, apInOut is not touched. It will be 
    //  returned unmodified.
    VIDEOSYSTEM3_API int vsDecompressImageInPlace
        (
        DIMAGE *apInOut 
        );

    //
    // The purpose of this function is to uncompress a whole sequence of
    // images at once (compressed image format -> raw image format). It 
    // uses vsDecompressImage() internally.
    //
    // Parameters:
    //   aImages - bunch of images that need to be decompressed
    //   aReleaseMemory - if set to 1(true), after one of the contained images
    //                    has been successfully decompressed, the corres-
    //                    ponding memory block which holds the compressed 
    //                    bits will be released (deallocated) using C api 
    //                    free() function
    //
    // Return code:
    //   1  - success - all images are now decompressed (raw format)
    //   0  - failure - no images in bunch aImages or aImages==NULL
    //  -1  - failure - error on decompression of some image in bunch, 
    //        aImages bunch may be partially altered, but should be 
    //        consistent (some images may have been decompressed, others
    //        should be left untouched)
    //
    // Remarks:
    //   1. For each image in bunch it is checked whether it is compressed.
    //      Currently, only JPEG (as if it would be a file) format is 
    //      supported. Any image in bunch that is not JPEG is simply 
    //      skipped.
    //
    VIDEOSYSTEM3_API int vsDecompressSequence
        (
        VSBUNCHOFIMAGES *aImages, 
        int aReleaseMemory 
        );


    // #ifdef __CORE_H_INCLUDED_SW__ this function is only available if videosystem 2 videokernel functions are available
#ifdef __CORE_H_INCLUDED_SW__

    //
    // This function performs exactly the same functionality as 
    // vsBunchToVIDINFHDREx, if special parameter aiLowMemoryConsumption is 
    // set to zero and apiOutImagesBunchUntouched is set to NULL.
    //
    VIDEOSYSTEM3_API int vsBunchToVIDINFHDR
        ( 
        const VSBUNCHOFIMAGES * const aImages, 
        struct VIDINFHDR *aV 
        );

    //
    // This function performs exactly the same functionality as 
    // vsBunchToVIDINFHDREx2, if parameter
    // asErrorStringReturnBuffer is NULL and 
    // anErrorReturnBufferSizeBytes is zero.
    //
    VIDEOSYSTEM3_API int vsBunchToVIDINFHDREx
        ( 
        VSBUNCHOFIMAGES *aImages, 
        struct VIDINFHDR *aV,
        int aiLowMemoryConsumption,
        int *apiOutImagesBunchUntouched
        );

    //
    //
    // In order to be able to use functions which have been taken over from
    // videokernel (open source API/library for videosystem 2), a converter 
    // function vsBunchToVIDINFHDREx2 is provided. It transcodes a bunch of 
    // images (VSBUNCHOFIMAGES) to the old format (VIDINFHDR). As the old 
    // format implies some constraints (see remarks), a proper checking of 
    // the source bunch will be performed before transcoding is started.
    //
    // Remarks:
    //   1. If transcoding has been finished successfully, behind 'aV' exist
    //      a cloned copy (new memory has been allocated) of all images 
    //      inside 'aImages'.
    //   2. constraints on image in order for the function to succeed:
    //      - aImages must be a sequence (complete sequence is not required)
    //      - image format of aImages must be grayscale with either 1 or 2
    //        bytes per pixel
    //      - if scale factors are present, xScale must be equal to yScale
    //        (because there exist only one scale for X and Y in VIDINFHDR)
    //   3. This alogrithm may not work on aImages which contain area of 
    //      interest (may work but untested).
    //
    // Parameters:
    //    aImages 
    //                           - VSBUNCHOFIMAGES filled with a proper image sequence
    //    aV      
    //                           - empty VIDINFHDR structure where the resulting transcoded
    //                             images will be put
    //
    //    char * asErrorStringReturnBuffer 
    //                           - user can supply a cstring buffer where a human 
    //                             readable error message will be stored in case
    //                             of unsuccessful return code.
    //
    //                             This parameter can be NULL.
    //
    //    unsigned int anErrorReturnBufferSizeBytes
    //                           - in case user supplied a cstring buffer at 
    //                             asErrorStringReturnBuffer, user must supply
    //                             here the size of the string buffer in bytes.
    //                             If the buffer is too small, the error string
    //                             will be cut. If the buffer is smaller than 2
    //                             bytes, nothing will be put in the buffer. It 
    //                             is recommended to supply a buffer size of 
    //                             200 bytes.
    // 
    //    int aiLowMemoryConsumption 
    //                           - must be set to zero or one
    //                           - if set to zero aImages will not be freed
    //                           - if set to one, aImages will be freed
    //                             frame by frame for each successfully 
    //                             converted frame
    //
    //    int * apiOutImagesBunchUntouched 
    //                           - in case of aiLowMemoryConsumption=1,
    //                             one can find out if aImages structure
    //                             has not been altered
    //
    //                             1: it has not been altered
    //                             0: it has been altered
    //
    //                             This parameter can be NULL.
    //              
    //
    // Return code: 
    //   1 - success, aV contains cloned copy
    //   0 - parameter error (aV NULL or not empty VIDINFHDR)
    //  -1 - unable to transcode (aImages malformed, not a sequence)
    //  -2 - unable to transcode (image format is not possible to transcode
    //       to VIDINFHDR, e.g. xScale != yScale)
    //  -3 - unable to transcode (memory allocation error)
    //
    // Remarks:
    //   1. In case of error return (return code < 1), if parameter 
    //      aiLowMemoryConsumption was set to one, apiImagesBunchUntouched
    //      is returned as '1' in order to indicate that aImages image 
    //      bunch structure has not been changed. If aImages structure has 
    //      been altered, '0' will be returned. 
    //
    //   2. In case aImages image bunch has been altered, it will be fully 
    //      freed before function returns.
    //
    //   3. It is guaranteed that aImages structure will only be changed if 
    //      parameter aiLowMemoryConsumption is set to one.
    //
    //   4. If aiLowMemoryConsumption is set to 1, for each frame 
    //      successfully converted from aImages to aV, frame memory at 
    //      aImages will be freed. If for example 1000 frames of 2.7 MB (uses 
    //      about 2.7 GB of RAM) needs to be converted, if vsBunchToVIDINFHDR 
    //      is used, 5.4 GB of RAM is required. If vsBunchToVIDINFHDREx with 
    //      aiLowMemoryConsumption set to 1 is used, only about 2.8 GB of RAM is
    //      required. In this case, frame bits at aImages will be no longer 
    //      available.
    //
    VIDEOSYSTEM3_API int vsBunchToVIDINFHDREx2
    (
        VSBUNCHOFIMAGES *aImages,
        struct VIDINFHDR *aV,
        char *asErrorStringReturnBuffer,
        unsigned int anErrorReturnBufferSizeBytes,
        int aiLowMemoryConsumption,
        int *apiOutImagesBunchUntouched
    );


#endif 

    //
    // vsLibraryInfo returns release versioning information. In any case 
    // it will return the major and minor release number of the shared 
    // library to the caller. 
    // If an appropriate string buffer is passed (absolute requirement is
    // that aBuildInfoStringBufferLengthBytes is bigger than 1), a version 
    // string will be returned. A version string
    // looks like this (without quotation marks): "*** VSv3 shared client 
    // library, win32(msvc6,x86), v0.9, b22(Release) (Jan 17, 2012) ***".
    // If one likes to find out how big the string buffer needs to 
    // be, one can pass NULL to parameter aBuildInfoString and will get 
    // the minimal size of string buffer returned via 
    // aBuildInfoStringBufferLengthBytes parameter.
    //
    // Parameters:
    //  aVersionMajor - [out] major version number of library
    //  aVersionMinor - [out] minor version number of library
    //  aBuildInfoString - [out] memory address of buffer which will receive
    //                     build info string or NULL
    //  aBuildInfoStringBufferLengthBytes - [in/out] capacity of build info
    //                                      string buffer in bytes
    //
    // Return value:
    //  aBuildInfoString - buffer has been successfully filled with data
    //  NULL             - buffer has not been filled, one can find its
    //                     minimal size at parameter 
    //                     aBuildInfoStringBufferLengthBytes
    //
    // Remarks:
    //   1. If aBuildInfoStringBufferLengthBytes is too small, 
    //      aBuildInfoStringBufferLengthBytes is filled as much as
    //      possible.
    //   2. if NULL is passed to aBuildInfoString, the function returns the
    //      minimum number of bytes required to fit the build info string
    //   3. A fixed buffer size of 150 bytes (chars) is enough, as of 
    //      Sept 22, 2015.
    //
    VIDEOSYSTEM3_API char * vsLibraryInfo
        (
        unsigned int *aVersionMajor, //updated by developer
        unsigned int *aVersionMinor, //updated by developer
        char *aBuildInfoString, 
        unsigned int *aBuildInfoStringBufferLengthBytes
        );

    VIDEOSYSTEM3_API char * vsLibraryInfoEx
        (
        unsigned int *aVersionMajor, //updated by developer
        unsigned int *aVersionMinor, //updated by developer
        unsigned int *aVersionSubMinor, //updated by developer
        char *aBuildInfoString, 
        unsigned int *aBuildInfoStringBufferLengthBytes
        );

    //
    // vsGetAllRawVideoServerNames returns a list of all servers which 
    // provide raw video data (Small Grabber Part (SGP) servers). 
    //
    // Parameters: 
    //  aSyslistFilenamePath - filename and path to syslist file
    //                         
    //  aOutTineContext      - on true return, single NAME64 structure will be 
    //                         filled with Tine Context Name (used for 
    //                         accessing servers)
    //  aOutTineServerNames  - on input, one needs to pass an array to NAME64, 
    //                         NULL is allowed
    //  aInOutNumServerNames - on input, one needs to pass number of elements 
    //                         of array aOutTineServerNames
    //                       - on true return, if aOutTineServerNames has been 
    //                         passed non-NULL on input, the variable will 
    //                         inform how many elements of array 
    //                         aOutTineServerNames have been 'filled with data'
    //                       - on true return, if aOutTineServerNames has been
    //                         passed NULL on input, the variable will inform 
    //                         how many elements of array aOutTineServerNames 
    //                         are required to get all server names
    //
    // Return code:
    //  -2 failure: internal error querying syslist
    //  -1 failure: could not successfully load syslist (file not found, file 
    //              contents are not wellformed, ...)
    //   0 failure: argument error (e.g. aSyslistFilenamePath passed NULL)
    //   1 success
    //
    VIDEOSYSTEM3_API int vsGetAllRawVideoServerNames
        ( 
        const char *aSyslistFilenamePath, 
        NAME64 *aOutTineContext, 
        NAME64 *aOutTineServerNames, 
        unsigned int *aInOutNumServerNames
        );


    //
    // vsGetAllImageSourceNames returns a list of all camera names the Syslist 
    // file contains. 
    //
    // Parameters: 
    //  aSyslistFilenamePath - filename and path to syslist file
    //                         
    //  aOutCameraNames      - on true return, the NAME64 array passed will be 
    //                         filled with up to aInOutNumCameras elements, NULL 
    //                         is allowed
    //
    //  aOutCameraNameIDs    - on true return, the NAME64I array passed will be 
    //                         filled with up to aInOutNumCameras elements (Name 
    //                         of camera and integer ID of camera), NULL is 
    //                         allowed
    //
    //  aInOutNumCameras     - on input, one needs to pass number of elements 
    //                         of array aOutCameraNames and/or aOutCameraNameIDs
    //                         available
    //                       - on true return, if aOutCameraNames and/or 
    //                         aOutCameraNameIDs has been passed non-NULL on 
    //                         input, the variable will inform how many array 
    //                         elements have actually been 'filled with data'
    //                       - on true return, if aOutCameraNames AND 
    //                         aOutCameraNameIDs has been passed NULL on input, 
    //                         the variable will inform 
    //                         how many array elements are required to get all 
    //                         cameras returned
    //
    // Return code:
    //  -2 failure: internal error querying syslist
    //  -1 failure: could not successfully load syslist (file not found, file 
    //              contents are not wellformed, ...)
    //   0 failure: argument error (e.g. aSyslistFilenamePath passed NULL)
    //   1 success
    //
    VIDEOSYSTEM3_API int vsGetAllImageSourceNames
        (
        const char *aSyslistFilenamePath,
        NAME64 *aOutCameraNames,
        NAME64I *aOutCameraNameIDs,
        unsigned int *aInOutNumCameras
        );


    //
    // Support function for Labview which is meant as an upgrade to 
    //
    //  VS_EXPORT INT32 getFrameFromImageBlob
    //      (
    //          unsigned char *frame,
    //          const unsigned char *imageBlob,
    //          int *frameSize,
    //          int *frameWidth, 
    //          int *frameHeight
    //      );
    //
    // which was part of videosystem.dll (part of videosystem 2).
    //
    //
    // Parameters:
    //  aInOutFrame                 - memory location where the decoded image
    //                                bits are stored
    //                              - only on successful return code the 
    //                                memory buffer will be altered
    //
    //  aInOutFrameSizeBytes        - on input: how big is the memory buffer
    //                                at aInOutFrame
    //                              - on output: how many bytes of the memory 
    //                                buffer have been used OR in case of 
    //                                return code 23 (TINE dimension_error) it
    //                                will be returned how big the buffer 
    //                                needs to be (in bytes)
    //
    //  aInDIMAGEBuffer             - image package returned via TINE (DIMAGE
    //                                with appended frameBuffer)
    //
    //  aOutFrameWidthInPixels 	    - width in pixels of the returned image
    //                                bits rectangle
    //                              - value will only be returned in case of
    //                                success (return code: 0)
    //                              - NULL may be passed to indicate that 
    //                                this value is not needed
    //
    //  aOutFrameHeightInPixels     - height in pixels of the returned image
    //                                bits rectangle
    //                              - value will be returned on return code 0 
    //                                (success)
    //                              - NULL may be passed to indicate that this
    //                                value is not needed
    //
    //  aOutRawDataFormat           - 0 -> 8 bits per pixel grayscale data
    //                              - 1 -> 16 bits per pixel grayscale data 
    //                                (little endian)
    //                              - 2 -> 24 bits per pixel RGB (8 bit red, 
    //                                8 bit green, 8 bit blue)
    //                              - value will be returned on return code 0 
    //                                (success)
    //                              - NULL may be passed to indicate that this
    //                                value is not needed
    //
    //  aOutBytesPerPixel           - 1 for 8 bits per pixel grayscale data
    //                              - 2 for 16 bits per pixel grayscale data
    //                              - 3 for 24 bits per pixel RGB data
    //                              - value will be returned on return code 0 
    //                                (success)
    //                              - NULL may be passed to indicate that this
    //                                value is not needed
    //
    //  aOutEffectiveBitsPerPixel	- the value gives information about how 
    //                                many bits of the  physical bits per 
    //                                pixel may contain data 
    //                              - on RGB data, aOutEffectiveBitsPerPixel 
    //                                will be always 24
    //                              - on 16 bit grayscale data, effective bits 
    //                                per pixel can range from 8 to 16 bits 
    //                                per pixel; usual values in practice are 
    //                                10, 12, 14, and 16
    //                              - on 8 bit grayscale data, effective bits
    //                                per pixel will be always 8
    //                              - value will be returned on return code 0 
    //                                (success)
    //                              - NULL may be passed to indicate that this
    //                                value is not needed
    //                              - a value of '-1' returned means that 
    //                                effective bits per pixel is not 
    //                                specified for this image
    //
    //  aOutDIMAGE                  - the DIMAGE structure (the input DIMAGE 
    //                                structure in general; changes are 
    //                                applied so that the structure resembles
    //                                the values returned (e.g. input image 
    //                                format may be JPEG, output image format 
    //                                returned in this DIMAGE here will be 
    //                                raw data then)
    //                              - There will be no data buffer (image 
    //                                bits) appended, just the header is 
    //                                returned.
    //                              - value will be returned on return code 0 
    //                                (success)
    //                              - NULL may be passed to indicate that this
    //                                value is not needed
    //
    // Return code:
    //   0                              - success
    //
    //   23 (TINE dimension_error)      - if aInOutFrame is too small for 
    //                                    holding the returned image bits
    //
    //   20 (TINE argument_list_error)  - input aInDIMAGEBuffer is NULL, 
    //                                    aInOutFrame is NULL, ...
    //
    //   103 (TINE invalid_data)        - input aInDIMAGEBuffer is invalid
    //
    //   513 (TINE, user-defined)       - error on decompression 
    //

    VIDEOSYSTEM3_API int vsGetFrameFromDIMAGE
        ( 
        void *aInOutFrame, 
        int *aInOutFrameSizeInBytes,
        const void *aInDIMAGEBuffer, 
        int *aOutFrameWidthInPixels, 
        int *aOutFrameHeightInPixels,
        int *aOutRawDataFormat,
        int *aOutBytesPerPixel,
        int *aOutEffectiveBitsPerPixel,
        void *aOutDIMAGE
        );


    // SW comment May 27, 2013 1/2 old fashioned name (without 'vs'), may be used already in the wild, 
    // SW comment May 27, 2013 2/2 so it's left in here for compatibility reasons
    // {
    VIDEOSYSTEM3_API int getFrameFromDIMAGE
        ( 
        void *aInOutFrame, 
        int *aInOutFrameSizeInBytes,
        const void *aInDIMAGEBuffer, 
        int *aOutFrameWidthInPixels, 
        int *aOutFrameHeightInPixels,
        int *aOutRawDataFormat,
        int *aOutBytesPerPixel,
        int *aOutEffectiveBitsPerPixel,
        void *aOutDIMAGE
        );
    // }


    // 
    // Return value:
    //    1     success
    //    0     buffer too small
    //   -1     parameter error
    //    
    // Remark:
    //   - recommended size of buffer 1024 bytes
    //   - added Sept 3, 2013
    VIDEOSYSTEM3_API int vsGetVerboseEx
        ( 
        int *apiOutVerboseLevel, 
        char *apsOutLogFileNamePathBuffer, 
        int *apiInOutLogFileNamePathBufferSizeBytes
        );

    //
    // Parameter
    //   aVerboseLevel:   0 no output, keep quiet
    //                    1 full verbose to stderr
    //                    2 full verbose to stdout
    //
    // return value:
    //   0  error setting aVerboseLevel (out of bounds)
    //   1  success
    //
    // Remarks:
    //   In case of 'full verbose to stdXXX', output will be 
    //   written to stdXXX at certain places in code.
    //
    VIDEOSYSTEM3_API int vsSetVerbose
        (
        int aVerboseLevel
        );


    //
    // returns the currently set verbose level
    // function introduced Jan 2013
    //
    // currently the verbose levels are 
    //  0 - keep quiet
    //  1 - full verbose to stderr
    //  2 - full verbose to stdout
    //  3 - full verbose to special file pointer passed to vsSetVerboseEx (lib >0.98)
    //
    VIDEOSYSTEM3_API int vsGetVerbose();


    // lib > 0.98
    // SW Apr 26, 2013 idea is to pass from matlab down to the library 
    // the matlab stdout file pointer in order
    // to let the library its verbosity placed to matlab command window

    //
    // Parameter
    //  aVerboseLevel:  0 same like vsSetVerbose (additional parameter not taken into account)
    //                  1 same like vsSetVerbose (additional parameter not taken into account)
    //                  2 same like vsSetVerbose (additional parameter not taken into account)
    //                  3 full verbose to FILE *aFilePath
    //
    // Remarks:
    //  - aFileNamePath may contain up to 255 characters (platform dependent path separation)
    //  - aFileNamePath must contain full filename at least, it can be extended by a absolute or relative path
    //  - if aFileNamePath contains empty string (""), default logging file name and path pattern will be initialized
    //    (environment variable AVINE_CLIENTLIB_LOGGING_PATH, fallback to platformGetDefaultLogFilePath() (/tmp on unix, 
    //     environment variable TEMP on windows), 
    //     filename-pattern: "avine-sharedlib--$year-$mon-$day--p-$processid--$u-$username--h-$hostname.log")
    //
    VIDEOSYSTEM3_API int vsSetVerboseEx
        (
        int aVerboseLevel,
        const char *aFileNamePath
        );


    // Except for a little extension of functionality (see below), the 
    // functions save_imm_fileEx, save_imc_fileEx and save_bkc_fileEx contain
    // the same functionality as similar functions from Video System 2, which
    // are still available: save_imm_file, save_imc_file and save_bkc_file. 
    // Documentation of these Video System 2 functions can be found at 
    // http://adweb.desy.de/mcs/tine/VideoSystem/vsv2doc/videokernel_v1_doc/vk_doc_v1.html#funcref
    //
    // In the following, only differences will be explained.
    // 
    // Added Parameters:
    //   apiInInterrupt - argument pointer to integer, input parameter
    //                  - apiInterrupt can be NULL, then there is no 
    //                    possibility to interrupt function 
    //                  - *apiInterrupt: set by remote thread:
    //                    1 to interrupt function (stop and return)
    //                    0 to not interrupt function (keep running)
    //                  - *apiInterrupt has to be set to '0' before
    //                    function is called
    //
    //   apdOutPercentageFinished - argument pointer to double, output 
    //                              parameter
    //                            - a remote thread can read the double 
    //                              this pointer points at
    //                            - the double will contain percentage 
    //                              the function has already finished 
    //                              (savedframes / totalframes)
    //                            - *apdPercentageFinished: 0..1 (0%.. 100%)
    //                            - apdPercentageFinished can be NULL, then
    //                              there is no possiblity to read how much 
    //                              of the saving is finished already
    // 
    // Added Return Value: 
    //   -9  user break 
    //        - (*apiInInterrupt=1)
    //        - incomplete file has been deleted
    // 
    // 
    VIDEOSYSTEM3_API INT32 save_imm_fileEx
        (
        const char *outfilename, 
        const struct VIDINFHDR *vinf,
        int *apiInInterrupt,
        double *apdOutPercentageFinished
        );

    VIDEOSYSTEM3_API INT32 save_bkc_fileEx
        (
        const char *outfilename, 
        const struct VIDINFHDR *vinf, 
        int *apiInInterrupt,
        double *apdOutPercentageFinished
        );

    VIDEOSYSTEM3_API INT32 save_imc_fileEx
        (
        const char *outfilename, 
        const struct VIDINFHDR *vinf,
        int *apiInInterrupt,
        double *apdOutPercentageFinished
        );




    //
    // Parameter
    //   aCC:						tine code
    //   aOutStringBuffer:			memory location where to put string representation
    //   aInOutStringBufferLength:  - on input, size in bytes of memory location 
    //                                (must be >=1 byte, recommended >= 32 bytes)
    //                              - on successful return, it outputs number of bytes used
    //                                for string (including null byte at the end)
    //
    // return value:
    //   0	error (parameter error)
    //   1	success
    //
    VIDEOSYSTEM3_API INT32 vsTineCodeToString
        ( 
        int aCC, 
        char *aOutStringBuffer, 
        size_t *aInOutStringBufferLength
        );

    // vsCheckImageValidity checks a DIMAGE structure whether its header fields and appended
    // frame buffer contain a valid image (header fields are correct and match image bits
    // appended in buffer).
    //
    // Parameter
    //   apImg:						   - image passed by pointer reference (must not be NULL)
    //   aOutStringBuffer:			   - memory location where to put human readable string 
    //                                   which contains error message in case of return value <1
    //
    //   anOutErrorStringBufferLength: - must contain number of bytes of aOutStringBuffer capacity (including nullbyte),
    //                                   recommended to be 350 bytes (349 characters plus nullbyte)
    // return value: 
    //  -1  error validating
    //   0	error (parameter error)
    //   1	success
    //

    VIDEOSYSTEM3_API INT32 vsCheckImageValidity(
        const DIMAGE *apImg,
        char *asOutErrorStringBuffer,
        size_t anOutErrorStringBufferLength
    );


    //
    // int vsLoadImagesFromFileEx()
    //
    // Parameters:
    //  asFilePath                   - path and file name to file out of which the
    //                                 image sequence apImages will be loaded 
    //                                 from, file extension does not matter
    //  apImages                     - pointer to struct VSBUNCHOFIMAGES, must be
    //                                 empty when passed (e.g. by vsInitialize-
    //                                 BunchStructure, or after vsFreeBunch()), 
    //                                 memory for images will be allocated 
    //                                 internally)
    //  asErrorStringReturnBuffer    - pointer to string buffer (character se-
    //                                 quence) which may be filled in case return 
    //                                 code is < 0. Can be NULL.
    //  anErrorReturnBufferSizeBytes - number of bytes, recommended to be at least 
    //                                 512 bytes, can be zero, in which case 
    //                                 asErrorStringReturnBuffer is not filled at all
    //
    //  apiInInterrupt               - argument pointer to integer, input parameter
    //                               - apiInterrupt can be NULL, then there is no 
    //                                 possibility to interrupt function 
    //                               - *apiInterrupt: set by remote thread:
    //                                 1 to interrupt function (stop and return)
    //                                 0 to not interrupt function (keep running)
    //                               - *apiInterrupt has to be set to '0' before
    //                                 function is called
    //
    //  apdOutPercentageFinished     - argument pointer to double, output 
    //                                 parameter
    //                               - a remote thread can read the double 
    //                                 this pointer points at
    //                               - the double will contain percentage 
    //                                 the function has already finished 
    //                                 (loadedframes / totalframes)
    //                               - *apdPercentageFinished: 0..1 (0%.. 100%)
    //                               - apdPercentageFinished can be NULL, then
    //                                 there is no possiblity to check how much 
    //                                 of the saving is finished already
    //
    // Return Code:
    //   0   ok
    //  -1   error on parameter validation (function parameters)
    //  -2   image/s not valid for loading (headers must be properly filled (_internal_vsCheckImageValidity failed) )
    //  -3   file structure malformed or no imc2(bkc2) file format
    //  -4   file could not be opened (for reading)
    //  -5   file read error (premature end of file)
    //  -6   internal error understanding metadata when reading from file
    //  -7   memory allocation error
    //  -8   zlib decompression failed
    //  -9   user break / interrupted by user
    //        - (*apiInInterrupt=1)
    // -10   internal error (size mismatch of structure)
    // -11   len_compressed or len_uncompressed too large (>int32_t max 0x7fffffff)
    //
    // -12   try to load as imm file failed
    // -13   try to load as bkg file failed
    // -14   try to load as imc/bkc file failed
    // -15   transcoding from VIDINFHDR (Video System 2) to VSBUNCHOFIMAGES (Video System 3) failed (load imm, bkg or imc/bkc)
    //

    VIDEOSYSTEM3_API INT32 vsLoadImagesFromFileEx(const char* asFilePath, VSBUNCHOFIMAGES* apImages, int* apiInInterrupt,
        double* apdOutPercentageFinished, char* asErrorStringReturnBuffer, unsigned int aiErrorReturnBufferSizeBytes);

    VIDEOSYSTEM3_API INT32 vsLoadImagesFromFile(const char* asFilePath, VSBUNCHOFIMAGES* apImages);



    // int vsSaveImagesToFileEx()
    //
    // Parameters:
    //  apImages                     - pointer to struct VSBUNCHOFIMAGES which is
    //                                 filled with a proper image sequence (1..n 
    //                                 images)
    //  asFilePath                   - path and file name to file in which the
    //                                 image sequence apImages will be saved.
    //                                 by file extension it will be checked
    //                                 which file format will be saved.
    //                                 - ".imc2" or ".bkc2" imc2(aka bkc2) file format
    //                                 - ".imc" or ".bkc" imc/bkc file format
    //                                 - ".imm": imm file format
    //                                 - ".bkg": bkg file format 
    //                                 The default file extension (if none is given) 
    //                                 is .imc2. The recommended extension is 
    //                                 '.imc2' for sequence of non- background images, 
    //                                 '.bkc2' for sequence of background images.
    //                                 Unless good reason, .imc2/bkc2 should be used for 
    //                                 saving, because it is most modern of the formats,
    //                                 and keeps properties of Video System 3 image header 
    //                                 (DIMAGE).
    //  asErrorStringReturnBuffer    - pointer to string buffer (character se-
    //                                 quence) which may be filled in case return 
    //                                 code is < 0. Can be NULL.
    //  anErrorReturnBufferSizeBytes - number of bytes, recommended to be at least 
    //                                 200 bytes, can be zero, in which case 
    //                                 asErrorStringReturnBuffer is not filled at all
    //
    //  apiInInterrupt               - argument pointer to integer, input parameter
    //                               - apiInterrupt can be NULL, then there is no 
    //                                 possibility to interrupt function 
    //                               - *apiInterrupt: set by remote thread:
    //                                 1 to interrupt function (stop and return)
    //                                 0 to not interrupt function (keep running)
    //                               - *apiInterrupt has to be set to '0' before
    //                                 function is called
    //
    //  apdOutPercentageFinished     - argument pointer to double, output 
    //                                 parameter
    //                               - a remote thread can read the double 
    //                                 this pointer points at
    //                               - the double will contain percentage 
    //                                 the function has already finished 
    //                                 (savedframes / totalframes)
    //                               - *apdPercentageFinished: 0..1 (0%.. 100%)
    //                               - apdPercentageFinished can be NULL, then
    //                                 there is no possiblity to check how much 
    //                                 of the saving is finished already
    //
    // Return Code:
    //
    //    0   ok
    //   -1   only single image can be saved to bkg file, not multiple images
    //   -2   internal error saving to imc/bkc/imm/bkg file: wrong vinf->type (neither TYPE_BMP, 
    //        TYPE_IMM, TYPE_IMC, TYPE_BKC, TYPE_BKG)
    //   -3   internal error saving to imc/bkc/imm/bkg file: vinf NULL or vinf->numframes is zero (no images to save)
    //  -12   error on parameter validation
    //  -13   images not valid for saving (must be sequence, headers must be properly filled and image format gray with 1 or 2 bytes
    //        per pixel or rgb24(3 bytes per pixel))
    //  -14   file already exists (file open "rb" did not fail) (only for .imc2/.bkc2)
    //   -4   file could not be opened for writing
    //   -5   file write error
    //  -15   internal error composing metadata when writing to file
    //   -6   memory allocation error
    //   -7   compression failed
    //   -9   user break 
    //        - (*apiInInterrupt=1)
    //        - incomplete file has been deleted
    //  -16   transcoding error (VSv3 images to VSV2 images structure as preparation for saving to imc/bkc/bkg/imm)
    //  
    VIDEOSYSTEM3_API INT32 vsSaveImagesToFileEx(VSBUNCHOFIMAGES* apImages,
        const char* asFilePath,
        char* asErrorStringReturnBuffer,
        unsigned int anErrorReturnBufferSizeBytes,
        int* apiInInterrupt,
        double* apdOutPercentageFinished
    );

    // identical to vsSaveImagesToFileEx, except no return code -9 (user break)
    VIDEOSYSTEM3_API INT32 vsSaveImagesToFile(VSBUNCHOFIMAGES* apImages,
        const char* asFilePath);

    // INT32 vsCheckFileFormat()
    //
    //  Checks whether image file is of a known good file format (IMM,BKG,IMC,BKC,IMC2 or BKC2. Contents of file are inspected,
    //   file extension does not matter.
    // 
    // Parameters:    
    //
    //   asFilePath                     - file name and path of existing file to be checked whether it is imm/bkg/imc/bkc/imc2/bkc2 file
    //   asOutErrorReturnString         - on negative return, human readable string explaining why file is no valid image file is returned,
    //                                    can be NULL (in this case no error string is returned)
    //   aiErrorReturnStringLengthBytes - number of bytes asOutErrorReturnString can hold (capacity), including nullbyte
    //                                    recommended to be at least 100 bytes, if smaller than two bytes, asOutErrorReturnString
    //                                    is not filled at all
    // Return Code:
    //    3   file is imc2/bkc2
    //    2   file is imc/bkc
    //    1   file is bkg
    //    0   file is imm
    //   -1   file format unknown or file could not be found or opened
    //
    VIDEOSYSTEM3_API INT32 vsCheckFileFormat(const char* asFilePath, char* asOutErrorReturnString, size_t aiErrorReturnStringLengthBytes);


    ///////////////////////////////////////////////////////////////////////////////////////////////
    //
    // INTERNAL EXPORTS
    //
    // FOR INTERNAL USE ONLY
    //
    // FUNCTION PROTOTYPES MAY CHANGE OR BE REMOVED WITHOUT ANY NOTICE
    //
    //

    VIDEOSYSTEM3_API char * VSV3_INTERNAL_CDIMAGE_getFormatAsString
        (
        int aFormat,
        char *aTextBuffer,
        size_t *aTextBufferLengthBytes
        );

    VIDEOSYSTEM3_API char * VSV3_INTERNAL_CDIMAGE_getFlagsAsString
        (
        int aFlags,
        char *aTextBuffer,
        size_t *aTextBufferLengthBytes
        );

    VIDEOSYSTEM3_API int VSV3_INTERNAL_tineDecodeModeStringCMnew
        (
        const char *aModeString
        ); // SW Jan 4, 2013 for matlab mex avine_tine_read_images

    // return value: 0 -> false -> error
    //               1 -> true -> success
    VIDEOSYSTEM3_API int VSV3_INTERNAL_tineCMtoCA
        (
        int aTineCMMode,
        int *aOutTineCAMode
        ); // SW Dec 21, 2015 for matlab mex avine_tine_read_images (vsGrabFrame for 1 frame)
    
    //
    // 
    // aTineMode   - tine transfer mode (integer with bits set or not set)
    // abSimpleMode - <>0: (true)   'CONNECT' etc.
    //                  0: (false)  'CM_CONNECT' etc.
    //
    // return value:
    //   1   success
    //   0   failure: conversion to string failed (usually this means passed aTineMode is illegal)
    //  -1   failure: text buffer is not big enough (aTextBufferLengthBytes has been adjusted to the size necessary)
    //
    VIDEOSYSTEM3_API int VSV3_INTERNAL_tineModeToString
        (
        int aTineMode, 
        int aSimpleMode, 
        char *aTextBuffer, 
        size_t *aTextBufferLengthBytes
        ); // SW Jan 8, 2013 for matlab mex avine_tine_read_images

    //
    // colortable
    //  - each index is unsigned char[4]
    //    idx[0] -> red 0..255 (no up to full intensity)
    //    idx[1] -> green 0..255 (no up to full intensity)
    //    idx[2] -> blue 0..255 (no up to full intensity)
    //    idx[3] - not used, should be set to zero
    //
    // colorTableType:
    //    0 gray
    //    1 pitz
    //    2 matlab jet (modified)
    //  <any other value> invalid parameter
    //
    // aPhysicalBytesPerPixel
    //   1 1 byte per pixel (8 bits) 
    //   2 2 bytes per pixel (up to 16 bits)
    // 
    // aEffectiveBitsPerPixel
    //  number of effective bits per pixel, usual value is 8 (256 distinct values will be created),
    //   10, 12, 14, 16 
    //  allowed are values from 8 to 16
    //  <any other value> invalid parameter
    //
    // aColorTableSizeBytes
    //   must not be NULL
    //   [in out]
    //   [in]: number of bytes in buffer aColorTable
    //   [out]: number of bytes in buffer which
    //          have been used, if return value >0
    //          are required, if return value == -2
    //
    // return value:
    //  
    //  >0 number of color entries that have been created 
    //
    //  -1 parameter error
    //  -2 aColorTableSizeBytes too small (correct value has been passed)
    //

    // SW Jan 18, 2013 implemented, but UNTESTED yet
    // SW Jan 18, 2013 foreseen for avine2mp4
    VIDEOSYSTEM3_API int VSV3_INTERNAL_vsCreateColourTable
        (
        unsigned char *aColorTable,
        int *aColorTableSizeBytes, 
        int aColorTableType,
        int aPhysicalBytesPerPixel,
        int aEffectiveBitsPerPixel
        );

    //
    // aInputBuffer 
    //  one dimensional array with the input data (gray values)
    //
    // aInputBufferSizeBytes
    //  size of the input buffer in bytes
    //
    // aOutputBuffer
    //  gray image transcoded (via color table) to RGB24 will have been put here
    //  on successful return 
    //
    // aInOutOutputBufferSizeBytes
    //  [in out]
    //  [in] number of bytes available in aOutputBuffer
    //  [out] number of bytes required in aOutputBuffer, if return -2
    //        number of bytes in output buffer that have been used, if return 0
    //
    // aColorTable
    //  pointer to a color table (each index 4 unsigned chars red, green, blue, unused)
    //  color table must have enough indices for aPhysicalBytesPerPixel[1..2] so 
    //  either 256 or 65536 entries 
    //
    // aPhysicalBytesPerPixel
    //  physical bytes per pixel, for color table as well as for input data
    //
    //
    // aNumPixels 
    //  number of pixels in input buffer (width*height, no padding, no gaps)
    //
    // 
    // return value
    //   0  success
    //  -1  parameter error
    //  -2  aInOutOutputBufferSizeBytes size too small
    //

    // SW Jan 18, 2013 implemented, but UNTESTED yet
    // SW Jan 18, 2013 foreseen for avine2mp4
    VIDEOSYSTEM3_API int VSV3_INTERNAL_vsGrayToRGB24
        (
        const void *aInputBuffer, 
        int aInputBufferSizeBytes, 
        void *aOutputBuffer,
        int *aInOutOutputBufferSizeBytes,
        const char *aColorTable,
        int aPhysicalBytesPerPixel,
        int aNumPixels
        );

    VIDEOSYSTEM3_API char *VSV3_INTERNAL_getTimeString
        ( 
        char *aOutTimeBuffer, 
        int aTimeBufferLengthBytes
        );

    // VSV3_INTERNAL_platformTimeToUTCString: wrap of VSV3 general src platform.cpp's platformTimeToUTCString()
    // (for Matlab mex avine_load_video_images_from_file.cpp etc.) 
    // 
    // buffer should be able to store 36 characters (including string terminating nullbyte)
    // returns NULL in case of problem during conversion
    //  - apOutStringBuffer is passed NULL
    //  - aiStringBufferLengthBytes is less than 2 bytes 
    //  - in case aibMicroseconds is != 0(true), anMilliOrMicroseconds is > 999999
    //  - in case aibMicroseconds is ==0(false), anMilliOrMicroseconds is > 999
    //  - in case gmtime is unsuccessful
    //  - in case tm_isdst is not zero (it should be zero after conversion to UTC time)
    //
    //           11111111112222222222333333
    //  123456789012345678901234567890123456
    // '[2016-03-28 14:28:59.312 UTC]' 
    // '[2016-03-28 12:20:53.742 PM UTC]' 
    // '[2016-03-28  2:20:53.743262 PM UTC]' 
    // '[2016-03-28 14:20:53.743262 UTC]' 
    VIDEOSYSTEM3_API char* VSV3_INTERNAL_platformTimeToUTCString(time_t atUTCseconds,
        uint32_t anMilliOrMicroseconds,
        int aibMicroseconds,
        int aibWithBrackets,
        int aibUse12HourFormat,
        char* apOutStringBuffer,
        unsigned int aiStringBufferLengthBytes);

    //VIDEOSYSTEM3_API void VSV3_INTERNAL_testFunction(); SW has been used Aug/Sept 2013 for tests

    // END OF INTERNAL EXPORTS
    //
    //
    ////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
}
#endif

#endif // __VSV3_CLIENT_API_H_INCLUDED_SW__
